@@ -1,4 +1,4 @@
-use crate::auditwheel::PlatformTag;
+use crate::auditwheel::{AuditWheelMode, PlatformTag};
 use crate::build_context::BridgeModel;
 use crate::compile::{CompileTarget, LIB_CRATE_TYPES};
 use crate::cross_compile::{find_sysconfigdata, parse_sysconfigdata};
@@ -185,8 +185,12 @@ pub struct BuildOptions {
     #[arg(short, long)]
     pub out: Option<PathBuf>,
 
+    /// Audit wheel for manylinux compliance
+    #[arg(long, conflicts_with = "skip_auditwheel")]
+    pub auditwheel: Option<AuditWheelMode>,
+
     /// Don't check for manylinux compliance
-    #[arg(long = "skip-auditwheel")]
+    #[arg(long, hide = true)]
     pub skip_auditwheel: bool,
 
     /// For manylinux targets, use zig to ensure compliance for the chosen manylinux version
@@ -270,6 +274,10 @@ impl BuildOptions {
                             .get("ABIFLAGS")
                             .map(ToString::to_string)
                             .unwrap_or_default();
+                        let gil_disabled = sysconfig_data
+                            .get("Py_GIL_DISABLED")
+                            .map(|x| x == "1")
+                            .unwrap_or_default();
                         let ext_suffix = sysconfig_data
                             .get("EXT_SUFFIX")
                             .context("syconfig didn't define an `EXT_SUFFIX` ಠ_ಠ")?;
@@ -295,6 +303,7 @@ impl BuildOptions {
                                 abiflags,
                                 ext_suffix: ext_suffix.to_string(),
                                 pointer_width: None,
+                                gil_disabled,
                             },
                             executable: PathBuf::new(),
                             platform: None,
@@ -372,6 +381,7 @@ impl BuildOptions {
                                 abiflags: "".to_string(),
                                 ext_suffix: ".pyd".to_string(),
                                 pointer_width: None,
+                                gil_disabled: false,
                             },
                             executable: PathBuf::new(),
                             platform: None,
@@ -398,6 +408,7 @@ impl BuildOptions {
                                 abiflags: "".to_string(),
                                 ext_suffix: ".pyd".to_string(),
                                 pointer_width: None,
+                                gil_disabled: false,
                             },
                             executable: PathBuf::new(),
                             platform: None,
@@ -437,6 +448,7 @@ impl BuildOptions {
                                 abiflags: "".to_string(),
                                 ext_suffix: "".to_string(),
                                 pointer_width: None,
+                                gil_disabled: false,
                             },
                             executable: PathBuf::new(),
                             platform: None,
@@ -483,12 +495,59 @@ impl BuildOptions {
 
     /// Tries to fill the missing metadata for a BuildContext by querying cargo and python
     #[instrument(skip_all)]
-    pub fn into_build_context(
-        self,
-        release: bool,
-        strip: bool,
-        editable: bool,
-    ) -> Result<BuildContext> {
+    pub fn into_build_context(self) -> BuildContextBuilder {
+        BuildContextBuilder::new(self)
+    }
+}
+
+#[derive(Debug)]
+pub struct BuildContextBuilder {
+    build_options: BuildOptions,
+    release: bool,
+    strip: bool,
+    editable: bool,
+    sdist_only: bool,
+}
+
+impl BuildContextBuilder {
+    fn new(build_options: BuildOptions) -> Self {
+        Self {
+            build_options,
+            release: false,
+            strip: false,
+            editable: false,
+            sdist_only: false,
+        }
+    }
+
+    pub fn release(mut self, release: bool) -> Self {
+        self.release = release;
+        self
+    }
+
+    pub fn strip(mut self, strip: bool) -> Self {
+        self.strip = strip;
+        self
+    }
+
+    pub fn editable(mut self, editable: bool) -> Self {
+        self.editable = editable;
+        self
+    }
+
+    pub fn sdist_only(mut self, sdist_only: bool) -> Self {
+        self.sdist_only = sdist_only;
+        self
+    }
+
+    pub fn build(self) -> Result<BuildContext> {
+        let Self {
+            build_options,
+            release,
+            strip,
+            editable,
+            sdist_only,
+        } = self;
         let ProjectResolver {
             project_layout,
             cargo_toml_path,
@@ -500,12 +559,15 @@ impl BuildOptions {
             mut cargo_options,
             cargo_metadata,
             mut pyproject_toml_maturin_options,
-        } = ProjectResolver::resolve(self.manifest_path.clone(), self.cargo.clone())?;
+        } = ProjectResolver::resolve(
+            build_options.manifest_path.clone(),
+            build_options.cargo.clone(),
+        )?;
         let pyproject = pyproject_toml.as_ref();
 
         let bridge = find_bridge(
             &cargo_metadata,
-            self.bindings.as_deref().or_else(|| {
+            build_options.bindings.as_deref().or_else(|| {
                 pyproject.and_then(|x| {
                     if x.bindings().is_some() {
                         pyproject_toml_maturin_options.push("bindings");
@@ -523,7 +585,7 @@ impl BuildOptions {
             );
         }
 
-        let mut target_triple = self.target.clone();
+        let mut target_triple = build_options.target.clone();
 
         let mut universal2 = target_triple.as_deref() == Some("universal2-apple-darwin");
         // Also try to determine universal2 from ARCHFLAGS environment variable
@@ -549,12 +611,14 @@ impl BuildOptions {
             };
         }
         if universal2 {
-            target_triple = None;
+            // Ensure that target_triple is valid. This is necessary to properly
+            // infer the platform tags when cross-compiling from Linux.
+            target_triple = Some("aarch64-apple-darwin".to_string());
         }
 
         let mut target = Target::from_target_triple(target_triple)?;
         if !target.user_specified && !universal2 {
-            if let Some(interpreter) = self.interpreter.first() {
+            if let Some(interpreter) = build_options.interpreter.first() {
                 if let Some(detected_target) =
                     crate::target::detect_arch_from_python(interpreter, &target)
                 {
@@ -563,38 +627,23 @@ impl BuildOptions {
             }
         }
 
-        let wheel_dir = match self.out {
+        let wheel_dir = match build_options.out {
             Some(ref dir) => dir.clone(),
             None => PathBuf::from(&cargo_metadata.target_directory).join("wheels"),
         };
 
         let generate_import_lib = is_generating_import_lib(&cargo_metadata)?;
-        let interpreter = if self.find_interpreter {
-            // Auto-detect interpreters
-            self.find_interpreters(
+        let interpreter = if sdist_only && env::var_os("MATURIN_TEST_PYTHON").is_none() {
+            // We don't need a python interpreter to build sdist only
+            Vec::new()
+        } else {
+            resolve_interpreters(
+                &build_options,
                 &bridge,
-                &[],
                 &target,
                 metadata23.requires_python.as_ref(),
                 generate_import_lib,
             )?
-        } else {
-            // User given list of interpreters
-            let interpreter = if self.interpreter.is_empty() && !target.cross_compiling() {
-                if cfg!(test) {
-                    match env::var_os("MATURIN_TEST_PYTHON") {
-                        Some(python) => vec![python.into()],
-                        None => vec![target.get_python()],
-                    }
-                } else {
-                    vec![target.get_python()]
-                }
-            } else {
-                // XXX: False positive clippy warning
-                #[allow(clippy::redundant_clone)]
-                self.interpreter.clone()
-            };
-            self.find_interpreters(&bridge, &interpreter, &target, None, generate_import_lib)?
         };
 
         if cargo_options.args.is_empty() {
@@ -607,11 +656,19 @@ impl BuildOptions {
         }
 
         let strip = pyproject.map(|x| x.strip()).unwrap_or_default() || strip;
-        let skip_auditwheel =
-            pyproject.map(|x| x.skip_auditwheel()).unwrap_or_default() || self.skip_auditwheel;
-        let platform_tags = if self.platform_tag.is_empty() {
+        let skip_auditwheel = pyproject.map(|x| x.skip_auditwheel()).unwrap_or_default()
+            || build_options.skip_auditwheel;
+        let auditwheel = build_options
+            .auditwheel
+            .or_else(|| pyproject.and_then(|x| x.auditwheel()))
+            .unwrap_or(if skip_auditwheel {
+                AuditWheelMode::Skip
+            } else {
+                AuditWheelMode::Repair
+            });
+        let platform_tags = if build_options.platform_tag.is_empty() {
             #[cfg(feature = "zig")]
-            let use_zig = self.zig;
+            let use_zig = build_options.zig;
             #[cfg(not(feature = "zig"))]
             let use_zig = false;
             let compatibility = pyproject
@@ -644,7 +701,7 @@ impl BuildOptions {
                 Vec::new()
             }
         } else {
-            self.platform_tag
+            build_options.platform_tag
         };
 
         for platform_tag in &platform_tags {
@@ -669,7 +726,7 @@ impl BuildOptions {
             );
         }
 
-        let target_dir = self
+        let target_dir = build_options
             .cargo
             .target_dir
             .clone()
@@ -697,9 +754,9 @@ impl BuildOptions {
             out: wheel_dir,
             release,
             strip,
-            skip_auditwheel,
+            auditwheel,
             #[cfg(feature = "zig")]
-            zig: self.zig,
+            zig: build_options.zig,
             platform_tag: platform_tags,
             interpreter,
             cargo_metadata,
@@ -710,6 +767,43 @@ impl BuildOptions {
     }
 }
 
+fn resolve_interpreters(
+    build_options: &BuildOptions,
+    bridge: &BridgeModel,
+    target: &Target,
+    requires_python: Option<&VersionSpecifiers>,
+    generate_import_lib: bool,
+) -> Result<Vec<PythonInterpreter>, anyhow::Error> {
+    let interpreter = if build_options.find_interpreter {
+        // Auto-detect interpreters
+        build_options.find_interpreters(
+            bridge,
+            &[],
+            target,
+            requires_python,
+            generate_import_lib,
+        )?
+    } else {
+        // User given list of interpreters
+        let interpreter = if build_options.interpreter.is_empty() && !target.cross_compiling() {
+            if cfg!(test) {
+                match env::var_os("MATURIN_TEST_PYTHON") {
+                    Some(python) => vec![python.into()],
+                    None => vec![target.get_python()],
+                }
+            } else {
+                vec![target.get_python()]
+            }
+        } else {
+            // XXX: False positive clippy warning
+            #[allow(clippy::redundant_clone)]
+            build_options.interpreter.clone()
+        };
+        build_options.find_interpreters(bridge, &interpreter, target, None, generate_import_lib)?
+    };
+    Ok(interpreter)
+}
+
 /// Checks for bridge/platform type edge cases
 fn validate_bridge_type(
     bridge: &BridgeModel,
@@ -1158,18 +1252,27 @@ fn find_interpreter_in_sysconfig(
     let mut interpreters = Vec::new();
     for interp in interpreter {
         let python = interp.display().to_string();
-        let (python_impl, python_ver) = if let Some(ver) = python.strip_prefix("pypy") {
-            (InterpreterKind::PyPy, ver.strip_prefix('-').unwrap_or(ver))
+        let (python_impl, python_ver, abiflags) = if let Some(ver) = python.strip_prefix("pypy") {
+            (
+                InterpreterKind::PyPy,
+                ver.strip_prefix('-').unwrap_or(ver),
+                "",
+            )
         } else if let Some(ver) = python.strip_prefix("graalpy") {
             (
                 InterpreterKind::GraalPy,
                 ver.strip_prefix('-').unwrap_or(ver),
+                "",
             )
         } else if let Some(ver) = python.strip_prefix("python") {
-            (
-                InterpreterKind::CPython,
-                ver.strip_prefix('-').unwrap_or(ver),
-            )
+            // Also accept things like `python3.13t` for free-threaded python
+            let (ver, abiflags) =
+                if let Some(ver) = ver.strip_prefix('-').unwrap_or(ver).strip_suffix('t') {
+                    (ver, "t")
+                } else {
+                    (ver, "")
+                };
+            (InterpreterKind::CPython, ver, abiflags)
         } else if python
             .chars()
             .next()
@@ -1177,7 +1280,7 @@ fn find_interpreter_in_sysconfig(
             .unwrap_or(false)
         {
             // Eg: -i 3.9 without interpreter kind, assume it's CPython
-            (InterpreterKind::CPython, &*python)
+            (InterpreterKind::CPython, &*python, "")
         } else {
             // if interpreter not known
             if std::path::Path::new(&python).is_file() {
@@ -1198,7 +1301,12 @@ fn find_interpreter_in_sysconfig(
         let ver_minor = ver_minor.parse::<usize>().with_context(|| {
             format!("Invalid python interpreter minor version '{ver_minor}', expect a digit")
         })?;
-        let sysconfig = InterpreterConfig::lookup_one(target, python_impl, (ver_major, ver_minor))
+
+        if (ver_major, ver_minor) < (3, 13) && abiflags == "t" {
+            bail!("Free-threaded Python interpreter is only supported on 3.13 and later.");
+        }
+
+        let sysconfig = InterpreterConfig::lookup_one(target, python_impl, (ver_major, ver_minor), abiflags)
             .with_context(|| {
                 format!("Failed to find a {python_impl} {ver_major}.{ver_minor} interpreter in known sysconfig")
             })?;
@@ -1361,7 +1469,7 @@ mod test {
     #[test]
     fn test_find_bridge_pyo3() {
         let pyo3_mixed = MetadataCommand::new()
-            .manifest_path(&Path::new("test-crates/pyo3-mixed").join("Cargo.toml"))
+            .manifest_path(Path::new("test-crates/pyo3-mixed").join("Cargo.toml"))
             .exec()
             .unwrap();
 
@@ -1378,7 +1486,7 @@ mod test {
     #[test]
     fn test_find_bridge_pyo3_abi3() {
         let pyo3_pure = MetadataCommand::new()
-            .manifest_path(&Path::new("test-crates/pyo3-pure").join("Cargo.toml"))
+            .manifest_path(Path::new("test-crates/pyo3-pure").join("Cargo.toml"))
             .exec()
             .unwrap();
 
@@ -1395,14 +1503,14 @@ mod test {
     #[test]
     fn test_find_bridge_pyo3_feature() {
         let pyo3_pure = MetadataCommand::new()
-            .manifest_path(&Path::new("test-crates/pyo3-feature").join("Cargo.toml"))
+            .manifest_path(Path::new("test-crates/pyo3-feature").join("Cargo.toml"))
             .exec()
             .unwrap();
 
         assert!(find_bridge(&pyo3_pure, None).is_err());
 
         let pyo3_pure = MetadataCommand::new()
-            .manifest_path(&Path::new("test-crates/pyo3-feature").join("Cargo.toml"))
+            .manifest_path(Path::new("test-crates/pyo3-feature").join("Cargo.toml"))
             .other_options(vec!["--features=pyo3".to_string()])
             .exec()
             .unwrap();
@@ -1416,7 +1524,7 @@ mod test {
     #[test]
     fn test_find_bridge_cffi() {
         let cffi_pure = MetadataCommand::new()
-            .manifest_path(&Path::new("test-crates/cffi-pure").join("Cargo.toml"))
+            .manifest_path(Path::new("test-crates/cffi-pure").join("Cargo.toml"))
             .exec()
             .unwrap();
 
@@ -1432,7 +1540,7 @@ mod test {
     #[test]
     fn test_find_bridge_bin() {
         let hello_world = MetadataCommand::new()
-            .manifest_path(&Path::new("test-crates/hello-world").join("Cargo.toml"))
+            .manifest_path(Path::new("test-crates/hello-world").join("Cargo.toml"))
             .exec()
             .unwrap();
 
@@ -1448,7 +1556,7 @@ mod test {
         assert!(find_bridge(&hello_world, Some("pyo3")).is_err());
 
         let pyo3_bin = MetadataCommand::new()
-            .manifest_path(&Path::new("test-crates/pyo3-bin").join("Cargo.toml"))
+            .manifest_path(Path::new("test-crates/pyo3-bin").join("Cargo.toml"))
             .exec()
             .unwrap();
         assert!(matches!(
