@@ -18,6 +18,7 @@ use fs_err::OpenOptions;
 use ignore::overrides::Override;
 use ignore::WalkBuilder;
 use indexmap::IndexMap;
+use itertools::Itertools;
 use normpath::PathExt as _;
 use same_file::is_same_file;
 use sha2::{Digest, Sha256};
@@ -185,7 +186,7 @@ impl ModuleWriter for PathWriter {
         target: impl AsRef<Path>,
         source: Option<&Path>,
         bytes: &[u8],
-        permissions: u32,
+        #[cfg_attr(target_os = "windows", allow(unused_variables))] permissions: u32,
     ) -> Result<()> {
         let path = self.base_path.join(&target);
 
@@ -971,7 +972,7 @@ struct UniFfiBindingsConfig {
 
 #[derive(Debug, Clone)]
 struct UniFfiBindings {
-    name: String,
+    names: Vec<String>,
     cdylib: String,
     path: PathBuf,
 }
@@ -984,7 +985,14 @@ fn uniffi_bindgen_command(crate_dir: &Path) -> Result<Command> {
         .no_deps()
         .verbose(true)
         .exec()?;
-    let root_pkg = cargo_metadata.root_package();
+    let root_pkg = match cargo_metadata.root_package() {
+        Some(pkg) => Some(pkg),
+        None => cargo_metadata
+            .packages
+            .iter()
+            .find(|p| p.manifest_path == manifest_path),
+    };
+
     let has_uniffi_bindgen_target = root_pkg
         .map(|pkg| {
             pkg.targets
@@ -1093,7 +1101,6 @@ fn generate_uniffi_bindings(
         bail!("Command {:?} failed", cmd);
     }
 
-    let py_binding = binding_dir.join(&py_binding_name).with_extension("py");
     // uniffi bindings hardcoded the extension filenames
     let cdylib_name = match cdylib_name {
         Some(name) => name,
@@ -1105,10 +1112,27 @@ fn generate_uniffi_bindings(
         _ => format!("lib{cdylib_name}.so"),
     };
 
+    let py_bindings = fs::read_dir(&binding_dir)?
+        .flatten()
+        .filter(|file| {
+            file.path()
+                .extension()
+                .and_then(std::ffi::OsStr::to_str)
+                .map_or(false, |ext| ext == "py")
+        })
+        .map(|file| {
+            file.path()
+                .file_stem()
+                .unwrap()
+                .to_string_lossy()
+                .to_string()
+        })
+        .collect_vec();
+
     Ok(UniFfiBindings {
-        name: py_binding_name,
+        names: py_bindings,
         cdylib,
-        path: py_binding,
+        path: binding_dir,
     })
 }
 
@@ -1126,11 +1150,16 @@ pub fn write_uniffi_module(
     pyproject_toml: Option<&PyProjectToml>,
 ) -> Result<()> {
     let UniFfiBindings {
-        name: binding_name,
+        names: binding_names,
         cdylib,
-        path: uniffi_binding,
+        path: binding_dir,
     } = generate_uniffi_bindings(crate_dir, target_dir, target_os, artifact)?;
-    let py_init = format!("from .{binding_name} import *  # NOQA\n");
+
+    let py_init = binding_names
+        .iter()
+        .map(|name| format!("from .{name} import *  # NOQA\n"))
+        .collect::<Vec<String>>()
+        .join("");
 
     if !editable {
         write_python_part(writer, project_layout, pyproject_toml)
@@ -1150,12 +1179,14 @@ pub fn write_uniffi_module(
             ))?;
 
             File::create(base_path.join("__init__.py"))?.write_all(py_init.as_bytes())?;
-            let target = base_path.join(&binding_name).with_extension("py");
-            fs::copy(&uniffi_binding, &target).context(format!(
-                "Failed to copy {} to {}",
-                uniffi_binding.display(),
-                target.display()
-            ))?;
+
+            for binding_name in binding_names.iter() {
+                let target: PathBuf = base_path.join(binding_name).with_extension("py");
+                fs::copy(binding_dir.join(binding_name).with_extension("py"), &target)
+                    .with_context(|| {
+                        format!("Failed to copy {:?} to {:?}", binding_dir.display(), target)
+                    })?;
+            }
         }
 
         let relative = project_layout
@@ -1181,10 +1212,12 @@ pub fn write_uniffi_module(
 
     if !editable || project_layout.python_module.is_none() {
         writer.add_bytes(module.join("__init__.py"), None, py_init.as_bytes())?;
-        writer.add_file(
-            module.join(binding_name).with_extension("py"),
-            uniffi_binding,
-        )?;
+        for binding in binding_names.iter() {
+            writer.add_file(
+                module.join(binding).with_extension("py"),
+                binding_dir.join(binding).with_extension("py"),
+            )?;
+        }
         writer.add_file_with_permissions(module.join(cdylib), artifact, 0o755)?;
     }
 
